"""This module contains the ServiceConfig class.
ServiceConfig is the base class for all service-related configurations.
"""

from abc import abstractmethod

from pydantic import BaseModel, ConfigDict

from mac.config.python_step import PythonStepConfig
from mac.config.service.server_config import ServerConfig
from mac.types import SupportedServices


class ServiceConfig(BaseModel):
    """This class represents the configuration for a Service object."""

    model_config = ConfigDict(arbitrary_types_allowed=True, extra="forbid")

    python_step: PythonStepConfig
    server: ServerConfig = ServerConfig()

    @property
    @abstractmethod
    def service_type(self) -> SupportedServices:
        """This property specifies the type of service this configuration is for."""
