"""This module defines custom types for the mac package."""

from enum import Enum
from typing import Callable, Dict

import numpy.typing as npt
from typing_extensions import TypeAlias

# PythonStep related types
InferenceData: TypeAlias = Dict[str, npt.NDArray]
PythonStep: TypeAlias = Callable[[InferenceData], InferenceData]


class SupportedFrameworks(str, Enum):
    """This class defines an Enum for supported frameworks. The frameworks are used
    to load a `PythonStep`.
    """

    KERAS = "keras"
    SKLEARN = "sklearn"
    PYTORCH = "pytorch"
    XGBOOST = "xgboost"
    HUGGING_FACE = "hugging-face"
    CUSTOM = "custom"  # BYOP
    PYTHON = "python"  # pre/post-processing step


class SupportedServices(str, Enum):
    """This class defines an Enum for supported services that
    can be used to serve a `PythonStep` for inference purposes.
    """

    MLFLOW = "mlflow"
    FLIGHT = "flight"
