from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.workspaces_validate_response_200_type_0 import (
    WorkspacesValidateResponse200Type0,
)
from ...models.workspaces_validate_response_400 import WorkspacesValidateResponse400
from ...models.workspaces_validate_response_401 import WorkspacesValidateResponse401
from ...models.workspaces_validate_response_500 import WorkspacesValidateResponse500
from ...types import Response


def _get_kwargs() -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/v1/api/workspaces/validate",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        Union["WorkspacesValidateResponse200Type0", None],
        WorkspacesValidateResponse400,
        WorkspacesValidateResponse401,
        WorkspacesValidateResponse500,
    ]
]:
    if response.status_code == HTTPStatus.OK:

        def _parse_response_200(
            data: object,
        ) -> Union["WorkspacesValidateResponse200Type0", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_0 = WorkspacesValidateResponse200Type0.from_dict(data)

                return response_200_type_0
            except:  # noqa: E722
                pass
            return cast(Union["WorkspacesValidateResponse200Type0", None], data)

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = WorkspacesValidateResponse400.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = WorkspacesValidateResponse401.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = WorkspacesValidateResponse500.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        Union["WorkspacesValidateResponse200Type0", None],
        WorkspacesValidateResponse400,
        WorkspacesValidateResponse401,
        WorkspacesValidateResponse500,
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[
    Union[
        Union["WorkspacesValidateResponse200Type0", None],
        WorkspacesValidateResponse400,
        WorkspacesValidateResponse401,
        WorkspacesValidateResponse500,
    ]
]:
    """Runs one-time migrations for workspaces that fail validation check.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Union['WorkspacesValidateResponse200Type0', None], WorkspacesValidateResponse400, WorkspacesValidateResponse401, WorkspacesValidateResponse500]]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[
    Union[
        Union["WorkspacesValidateResponse200Type0", None],
        WorkspacesValidateResponse400,
        WorkspacesValidateResponse401,
        WorkspacesValidateResponse500,
    ]
]:
    """Runs one-time migrations for workspaces that fail validation check.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Union['WorkspacesValidateResponse200Type0', None], WorkspacesValidateResponse400, WorkspacesValidateResponse401, WorkspacesValidateResponse500]
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[
    Union[
        Union["WorkspacesValidateResponse200Type0", None],
        WorkspacesValidateResponse400,
        WorkspacesValidateResponse401,
        WorkspacesValidateResponse500,
    ]
]:
    """Runs one-time migrations for workspaces that fail validation check.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Union['WorkspacesValidateResponse200Type0', None], WorkspacesValidateResponse400, WorkspacesValidateResponse401, WorkspacesValidateResponse500]]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[
    Union[
        Union["WorkspacesValidateResponse200Type0", None],
        WorkspacesValidateResponse400,
        WorkspacesValidateResponse401,
        WorkspacesValidateResponse500,
    ]
]:
    """Runs one-time migrations for workspaces that fail validation check.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Union['WorkspacesValidateResponse200Type0', None], WorkspacesValidateResponse400, WorkspacesValidateResponse401, WorkspacesValidateResponse500]
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed
